// Copyright (C) 2011 Helmut Hirtes
// This file is a free extension of the "Irrlicht Engine".
// For conditions of distribution and use, see copyright notice in irrlicht.h

#include "CGUIWindow.h"
#ifdef _IRR_COMPILE_WITH_GUI_

#include "CGUIFrame.h"
#include "IGUISkin.h"
#include "IGUIEnvironment.h"
#include "IVideoDriver.h"
#include "IGUIButton.h"
#include "IGUIFont.h"
#include "IGUIFontBitmap.h"
#include "IGUIRadioButton.h"

namespace irr
{
namespace gui
{

//! constructor
CGUIFrame::CGUIFrame(IGUIEnvironment* environment, IGUIElement* parent, core::rect<s32> rectangle, bool frame, s32 id)
: IGUIFrame(environment, parent, rectangle, id)
, HasFrame(frame)
, Index(0)
{
	#ifdef _DEBUG
	setDebugName("CGUIFrame");
	#endif
}


//! Gets the radio index.
s32 CGUIFrame::getIndex() const
{
  return Index;
}


//! Sets the radio index.
void CGUIFrame::setIndex(s32 index)
{
  Index = index;
  index = 0;
  // check all radio buttons
  core::list<IGUIElement*>::Iterator iter = Children.begin();
  for (; iter!=Children.end(); ++iter)
  if ((*iter)->hasType(EGUIET_CHECK_BOX))  // ### TBD: Set EGUIET_CHECK_BOX to new enum EGUIET_RADIO_BUTTON.
  {
    const bool checked = (index == Index);
    IGUIRadioButton &rb = *static_cast<IGUIRadioButton*>(*iter);
    if (rb.isChecked() != checked)
    {
      rb.setChecked( checked );
    }
    ++index;
  }
}


//! called if an event happened.
bool CGUIFrame::OnEvent(const SEvent& event)
{
	if (IsEnabled)
  if (event.EventType == EET_GUI_EVENT)
	{
		switch(event.GUIEvent.EventType)
		{
      case EGET_CHECKBOX_CHANGED:  // ### TBD: Set enum to EGET_RADIOBUTTON_CHANGED.
      { IGUIRadioButton &rb = *static_cast<IGUIRadioButton*>(event.GUIEvent.Caller);
        if (rb.isChecked())
        {
          const s32 index = findIndex(event.GUIEvent.Caller);
          setIndex( index );
          if (Parent)
          {
            SEvent newEvent;
            newEvent.EventType = EET_GUI_EVENT;
            newEvent.GUIEvent.Caller = this;
            newEvent.GUIEvent.Element = 0;
            newEvent.GUIEvent.EventType = EGET_CHECKBOX_CHANGED;  // ### TBD: Set enum to EGET_RADIOBUTTON_CHANGED.
            Parent->OnEvent(newEvent);
          }
        }
      } return true;
      default:
        break;
		}
	}
	return IGUIFrame::OnEvent(event);
}


//! draws the element and its children
void CGUIFrame::draw()
{
	if (IsVisible)
	{
	  // draw the frame
	  if (HasFrame)
	  {
      IGUISkin* skin = Environment->getSkin();
      core::rect<s32> clientClip(AbsoluteRect);
      core::rect<s32> frameRect(AbsoluteRect);
      skin->draw3DSunkenPane(this, skin->getColor(EGDC_3D_HIGH_LIGHT), true, false, frameRect, &clientClip);
	  }
	}
	IGUIFrame::draw();
}


void CGUIFrame::serializeAttributes(io::IAttributes* out, io::SAttributeReadWriteOptions* options) const
{
  out->addBool("HasFrame", HasFrame);
  out->addInt ("Index",    Index);
}


void CGUIFrame::deserializeAttributes(io::IAttributes* in, io::SAttributeReadWriteOptions* options)
{
	HasFrame = in->getAttributeAsBool("HasFrame");
	Index    = in->getAttributeAsInt ("Index");
}


s32 CGUIFrame::findIndex(IGUIElement* radio_button) const
{
  s32 index = 0;
  // loop over all radio buttons
  core::list<IGUIElement*>::ConstIterator iter = Children.begin();
  for (; iter!=Children.end(); ++iter)
  if ((*iter)->hasType(EGUIET_CHECK_BOX))  // ### TBD: Set EGUIET_CHECK_BOX to new enum EGUIET_RADIO_BUTTON.
  {
    const IGUIRadioButton &rb = *static_cast<const IGUIRadioButton*>(*iter);
    if (&rb == radio_button)
      break;
    ++index;
  }
  return index;
}


} // end namespace gui
} // end namespace irr

#endif // _IRR_COMPILE_WITH_GUI_

