// Copyright (C) 2011 Helmut Hirtes
// This file is a free extension of the "Irrlicht Engine".
// For conditions of distribution and use, see copyright notice in irrlicht.h
// Notes:
// - The methods constructor() and draw() are copied and modified from class CGUICheckBox.

#include "CGUIRadioButton.h"

#ifdef _IRR_COMPILE_WITH_GUI_

#include "IGUISkin.h"
#include "IGUIEnvironment.h"
#include "IGUIFont.h"
#include "os.h"

namespace irr
{
namespace gui
{

//! constructor
CGUIRadioButton::CGUIRadioButton(IGUIEnvironment* environment, IGUIElement* parent, core::rect<s32> rectangle, s32 id)
: CGUICheckBox(false, environment, parent, id, rectangle)
{
	#ifdef _DEBUG
	setDebugName("CGUIRadioButton");
	#endif

	// this element can be tabbed into
	setTabStop(true);
	setTabOrder(-1);
}


//! called if an event happened.
bool CGUIRadioButton::OnEvent(const SEvent& event)
{
  // ### TBD: If derived from CGUICheckBox we must ignore all deselections
  if (isChecked())
    return true;
	return CGUICheckBox::OnEvent(event);
}


//! draws the element and its children
void CGUIRadioButton::draw()
{
  const bool Pressed = false;  // ### TBD: We need access to our base class here!
  u32 checkTime = 0;  // ### TBD: We need access to our base class here!

	if (!IsVisible)
		return;

	IGUISkin* skin = Environment->getSkin();

	core::rect<s32> rect = AbsoluteRect;

	s32 height = skin->getSize(EGDS_CHECK_BOX_WIDTH);

	core::rect<s32> checkRect(AbsoluteRect.UpperLeftCorner.X,
				((AbsoluteRect.getHeight() - height) / 2) + AbsoluteRect.UpperLeftCorner.Y,
				0, 0);

	checkRect.LowerRightCorner.X = checkRect.UpperLeftCorner.X + height;
	checkRect.LowerRightCorner.Y = checkRect.UpperLeftCorner.Y + height;

	skin->draw3DSunkenPane(this, skin->getColor(Pressed || !IsEnabled ? EGDC_3D_FACE : EGDC_ACTIVE_CAPTION),
		false, true, checkRect, &AbsoluteClippingRect);

	if (isChecked() && Environment->getSkin())
		Environment->getSkin()->drawIcon(this, EGDI_RADIO_BUTTON_CHECKED, checkRect.getCenter(),
			checkTime, os::Timer::getTime(), false, &AbsoluteClippingRect);

	if (Text.size())
	{
		checkRect = AbsoluteRect;
		checkRect.UpperLeftCorner.X += height + 5;

		IGUIFont* font = skin->getFont();
		if (font)
			font->draw(Text.c_str(), checkRect,
					skin->getColor(IsEnabled ? EGDC_BUTTON_TEXT : EGDC_GRAY_TEXT), false, true, &AbsoluteClippingRect);
	}

	IGUIElement::draw();
}


} // end namespace gui
} // end namespace irr

#endif // _IRR_COMPILE_WITH_GUI_

